package com.tomwheeler.example.stringmaster.gui.resources;

import com.tomwheeler.example.stringmaster.api.StringFunction;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import javax.swing.DefaultComboBoxModel;
import org.openide.ErrorManager;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/**
 * Top component which displays something.
 */
final class StringConsoleTopComponent extends TopComponent {
    
    private static StringConsoleTopComponent instance;
    /** path to the icon used by the component and its open action */
    static final String ICON_PATH = "com/tomwheeler/example/stringmaster/gui/resources/stringmaster_window.gif";
    
    private static final String PREFERRED_ID = "StringConsoleTopComponent";
    
    private StringConsoleTopComponent() {
        initComponents();
        setName(NbBundle.getMessage(StringConsoleTopComponent.class, "CTL_StringConsoleTopComponent"));
        setToolTipText(NbBundle.getMessage(StringConsoleTopComponent.class, "HINT_StringConsoleTopComponent"));
        setIcon(Utilities.loadImage(ICON_PATH, true));
        
        
        // NOTE: my custom code is here
        functionCombo.setRenderer(new StringFunctionRenderer());
        
        DefaultComboBoxModel model = (DefaultComboBoxModel)functionCombo.getModel();
        model.removeAllElements();
        
        // behold the global lookup
        Lookup.Template tpl = new Lookup.Template (StringFunction.class); 
        Lookup.Result result = Lookup.getDefault().lookup (tpl); 
        Collection c = result.allInstances(); 
        for (Iterator i = c.iterator(); i.hasNext();) { 
            StringFunction sf = (StringFunction)i.next(); 
            model.addElement(sf);
        }
        
        // sanity check
        if (model.getSize() == 0) {
            invokeButton.setEnabled(false);
            throw new IllegalStateException("No StringFunctions Available!!!");
        }
        
        invokeButton.addActionListener(new ActionListener() {
           public void actionPerformed(ActionEvent e)  {
               String input = inputTextArea.getText();
               
               StringFunction func = (StringFunction)functionCombo.getSelectedItem();
               String output = func.invoke(input);
               
               outputTextArea.setText(output);
           }
        });
        
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jLabel1 = new javax.swing.JLabel();
        functionCombo = new javax.swing.JComboBox();
        jLabel2 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        inputTextArea = new javax.swing.JTextArea();
        jLabel3 = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        outputTextArea = new javax.swing.JTextArea();
        invokeButton = new javax.swing.JButton();

        org.openide.awt.Mnemonics.setLocalizedText(jLabel1, "Function:");

        functionCombo.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        org.openide.awt.Mnemonics.setLocalizedText(jLabel2, "Input:");

        inputTextArea.setColumns(20);
        inputTextArea.setRows(5);
        jScrollPane1.setViewportView(inputTextArea);

        org.openide.awt.Mnemonics.setLocalizedText(jLabel3, "Output:");

        outputTextArea.setColumns(20);
        outputTextArea.setRows(5);
        jScrollPane2.setViewportView(outputTextArea);

        org.openide.awt.Mnemonics.setLocalizedText(invokeButton, "Invoke Selected Function");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel1)
                            .add(jLabel2))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 66, Short.MAX_VALUE)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(jScrollPane1)
                            .add(functionCombo, 0, 294, Short.MAX_VALUE)))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(jLabel3)
                        .add(81, 81, 81)
                        .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 289, Short.MAX_VALUE))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, invokeButton))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(27, 27, 27)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(functionCombo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(25, 25, 25)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel2)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(20, 20, 20)
                .add(invokeButton)
                .add(23, 23, 23)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 95, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel3))
                .addContainerGap(52, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox functionCombo;
    private javax.swing.JTextArea inputTextArea;
    private javax.swing.JButton invokeButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextArea outputTextArea;
    // End of variables declaration//GEN-END:variables
    
    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link findInstance}.
     */
    public static synchronized StringConsoleTopComponent getDefault() {
        if (instance == null) {
            instance = new StringConsoleTopComponent();
        }
        return instance;
    }
    
    /**
     * Obtain the StringConsoleTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized StringConsoleTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            ErrorManager.getDefault().log(ErrorManager.WARNING, "Cannot find StringConsole component. It will not be located properly in the window system.");
            return getDefault();
        }
        if (win instanceof StringConsoleTopComponent) {
            return (StringConsoleTopComponent)win;
        }
        ErrorManager.getDefault().log(ErrorManager.WARNING, "There seem to be multiple components with the '" + PREFERRED_ID + "' ID. That is a potential source of errors and unexpected behavior.");
        return getDefault();
    }
    
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }
    
    public void componentOpened() {
        
    }
    
    public void componentClosed() {
        // TODO add custom code on component closing
    }
    
    /** replaces this in object stream */
    public Object writeReplace() {
        return new ResolvableHelper();
    }
    
    protected String preferredID() {
        return PREFERRED_ID;
    }
    
    final static class ResolvableHelper implements Serializable {
        private static final long serialVersionUID = 1L;
        public Object readResolve() {
            return StringConsoleTopComponent.getDefault();
        }
    }
}
